#include "MainWindow.hpp"
#include "DetailWindow.hpp"
#include "RSSReader.hpp"
#include "SoftkeyWindow.hpp"

#define SUBSURIPTION_MARGIN 30

DetailWindow::DetailWindow(SInt16 cursor) :
    SFRTitleWindow(SFRApplication::GetInstance(),
        SFXGraphics::GetDeviceRectangle().SubBottom(SoftkeyWindow::GetInstance()->GetHeight()),
        SFXAnsiString::EmptyInstance()),
    _displine(0), _cursor(0) static_throws
{
    // Drawing handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE,
                HANDLER_FUNCTION(OnRenderContent)));
    }
    // Key handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }
    // Calculate the number of lines
    _maxline = GetContentWorld().GetHeight()
        / SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) / 2;
    SetSoftkey();

    _cursor=cursor;
    _moveY = 0;
    UpdateTitle();


    return;
}

Void DetailWindow::SetSoftkey()
{
    SoftkeyWindow::GetInstance()->Set("Previous", "Return", "Next");
    return;
}

Void DetailWindow::UpdateTitle(Void)
{
    RSSFeedPtr current;
    current = RssReader::GetCurrentFeed();
    SFXAnsiString dTitle;
    dTitle = current->GetItem(_cursor)->title;
    SFRTitleWindow::SetTitle(dTitle);
    InvalidateBase();
    InvalidateContent();
}

HANDLER_IMPLEMENT_VOIDRENDER(DetailWindow, OnRenderContent, graphics)
{
    // Get the screen size
    SFXRectangle rectangle(GetContentWorld());
    SFXRGBColor forecolor;
    SFXRGBColor backcolor;
    RSSFeedPtr current;

    SInt16 height = graphics->GetFontHeight();
    SInt16 bottom = rectangle.GetBottom();

    // The rectangle height is set to the font height
    rectangle.SetHeight(height);

    SFRTitleWindow::ContentHandler(graphics);
    current = RssReader::GetCurrentFeed();

    rectangle.SetHeight(((bottom - rectangle.GetTop()) / height) * height);
    graphics->FillRectangle(rectangle, SFXRGBColor(255, 255, 255, 0));

    graphics->SetForeColor(SFXRGBColor(0, 0, 0, 0));
    if (current != null && current->GetSize() > _displine + _cursor) {
        _currentRect = rectangle;
        if (_currentRect.GetY() >= rectangle.GetY()) {
            _currentRect.SubY(_moveY);
        }
        _currentRect.AddHeight(500);

        // Draw the item description
        graphics->DrawString(current->GetItem(_displine + _cursor)->description,
            _currentRect, IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    }

    return;
}

HANDLER_IMPLEMENT_BOOLEVENT(DetailWindow, OnKey, event)
{
    SFXRectangle rectangle;
    switch (event.GetP16()) {

        // Scroll down
        case AVK_8:
        case AVK_DOWN:
            _moveY = _moveY + 5;
            InvalidateContent();
            return true;

        // Scroll up
        case AVK_2:
        case AVK_UP:
            if(_moveY > 0)
            {
                _moveY = _moveY - 5;
                InvalidateContent();
            }
            return true;

        // Detail window close
        case AVK_5:
        case AVK_CLR:
            Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
            MainWindow::SetSoftkey();
        return true;

        // If the cursor is at the bottom
        case AVK_RIGHT:
        case AVK_SOFT2:
            if (_cursor == _maxline - 1) {
                ++_displine;
                InvalidateContent();
            } else {
                ++_cursor;
                InvalidateContent();
            }
            _moveY = 0;
            UpdateTitle();
            return true;

        // If the cursor is at the top
        case AVK_LEFT:
        case AVK_SOFT1:
            if (_cursor == 0) {
                if (_displine > 0) {
                    --_displine;
                    InvalidateContent();
                }
            } else {
                --_cursor;
                InvalidateContent();
            }
            _moveY = 0;
            UpdateTitle();
            return true;
        }
    return false;
}
