
#include "InputWindow.hpp"
#include "SubscriptionWindow.hpp"
#include "SoftkeyWindow.hpp"
#include "RssReader.hpp"

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
InputWindow::InputWindow() : SFRTitleWindow(SFRApplication::GetInstance(),
    SFXGraphics::GetDeviceRectangle().Deflate(20, 20).SubBottom(15),
    "Edit") static_throws
{
    SFXRectangle rectangle = GetContentWorld();
    SInt16 height = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + 4;
    SInt16 space = height / 3;
    rectangle.SetHeight(height);
    rectangle.Offset(10, 10);

    // Label creation
    ::new SFRLabelControl(this, rectangle, "Title");
    rectangle.Offset(20, height + space);
    rectangle.SubWidth(35);

    // Title input control creation
    _titleTextControl = ::new SFREditboxControl(
        this, rectangle, SFXWideString::EmptyInstance());
    rectangle.Offset(-20, height + space);
    new SFRLabelControl(this, rectangle, "URL");
    rectangle.Offset(20, height + space);

	_titleTextControl->SetText("g");

    // URL input control creation
    _urlTextControl = new SFREditboxControl(
        this, rectangle, SFXWideString::EmptyInstance());
    rectangle.Offset(-20, height + space + 10);
    rectangle.SetWidth(50);
    rectangle.AddHeight(2);
	_urlTextControl->SetText("http://www.hitachi-hri.com/rss/word.xml");

    // Buttons creation
    SFRButtonControlPtr okButton = new SFRButtonControl(
        this, rectangle, "Register");
    rectangle.AddX(60);
    rectangle.SetWidth(120);
    SFRButtonControlPtr cancelButton = new SFRButtonControl(
        this, rectangle, "Cancel");

    // Focus to the head
    FocusNext();

    // Key handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }

    // Button handlers registration
    if (okButton != null && static_try()) {
        static_throw(
            okButton->RegisterHandler(
                SREVT_CONTROL, HANDLER_BEFORE,
                HANDLER_FUNCTION(OkButtonHandler)));
    }
    if (cancelButton != null && static_try()) {
        static_throw(
            cancelButton->RegisterHandler(
                SREVT_CONTROL, HANDLER_BEFORE,
                HANDLER_FUNCTION(CancelButtonHandler)));
    }

    // "Select" key press handler registration (for text controls)
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SFEVT_KEY, AVK_SELECT, HANDLER_AFTER,
                HANDLER_FUNCTION(SelectHandler)));
    }
    return;
}

//
// Soft key character string update.
//
Void InputWindow::SetSoftkey()
{
    // NULL string
    SoftkeyWindow::GetInstance()->Set("");
    return;
}

//
// Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(InputWindow, OnKey, event)
{
    switch (event.GetP16()) {
        case AVK_CLR:
        case AVK_SOFT2:
        case AVK_SOFT1:
            if (_titleTextControl->GetStatusTarget() || _urlTextControl->GetStatusTarget()){

                _titleTextControl->SetStatusTarget(false);
                _urlTextControl->SetStatusTarget(false);

                FocusNext();
            }
            else {
                // Close window
                Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,
                SRP16_TERMINATE_INVOKE, true));
                SubscriptionWindow::SetSoftkey();
                return true;
            }
        // Move focus
        case AVK_UP:
            FocusUp();
            return true;
        case AVK_LEFT:
            FocusLeft();
            return true;
        case AVK_DOWN:
            FocusDown();
            return true;
        case AVK_RIGHT:
            FocusRight();
            return true;
        default:
            break;
    }
    return false;
}

//
// OK button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(InputWindow, OkButtonHandler, result, control)
{
    unused(control);
    unused(result);

    // The title and URL are gathered from the text controls
    RSSFeedPtr feed = ::new RSSFeed(_titleTextControl->GetText(),
        _urlTextControl->GetText());
    if (feed != null) {
        // Error processing
        if (!RssReader::GetRSSFeedList()->Add(feed)) {
            ::delete feed;
        }
        else {
            // Save added data
            RssReader::GetRSSFeedList()->SaveToFile();
        }
    }

    // Dialog display
    SFRMessageDialogPtr dialog = ::new SFRMessageDialog(
        SFXGraphics::GetDeviceRectangle().Deflate(10, 100),
        "Registered", "OK");
    if (static_try()) {
        static_throw(dialog->RegisterHandler(
            SREVT_DIALOG, HANDLER_BEFORE, HANDLER_FUNCTION(EndHandler)));
    }
    SubscriptionWindow::SetSoftkey();
    return;
}

//
// "Cancel" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(InputWindow, CancelButtonHandler, result, control)
{
    unused(control);
    unused(result);

    SubscriptionWindow::SetSoftkey();

    // Close window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    return;
}

//
// Dialog OK button press handler.
//
HANDLER_IMPLEMENT_VOIDDIALOG(InputWindow, EndHandler, result, dialog)
{
    unused(dialog);
    unused(result);

    SubscriptionWindow::SetSoftkey();

    // Close InputWindow
    SFRWindowPtr window = static_cast<SFRWindowPtr>(GetPrevious());
    window->Invoke(
        SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));

    // Close dialog
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    return;
}
