
#include "MainWindow.hpp"
#include "RSSReader.hpp"
#include "SubscriptionWindow.hpp"
#include "SoftkeyWindow.hpp"
#include "DetailWindow.hpp"

#define SUBSCRIPTION_MARGIN 30

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
MainWindow::MainWindow(SInt16 softkeyHeight) :
    SFRTitleWindow(SFRApplication::GetInstance(),
        SFXGraphics::GetDeviceRectangle().SubBottom(softkeyHeight),
        SFXAnsiString::EmptyInstance()),
    _displine(0), _cursor(0) static_throws
{
     // Drawing handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE,
                HANDLER_FUNCTION(OnRenderContent)));
    }

    // Key handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }

    // Number of display lines calculation
    _maxline = GetContentWorld().GetHeight()
        / SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) / 2;
    SetSoftkey();

    _moveX = 0;
    _moveY = 0;
    return;
}

//
// Set the softkey strings.
//
void MainWindow::SetSoftkey()
{
    SoftkeyWindow::GetInstance()->Set("Finish", "Select", "List");
    return;
}

//
// Window drawing handler.
//
HANDLER_IMPLEMENT_VOIDRENDER(MainWindow, OnRenderContent, graphics)
{
    // Get size of screen
    SFXRectangle rectangle(GetContentWorld());
    SFXRGBColor forecolor;
    SFXRGBColor backcolor;
    SIntN i;
    RSSFeedPtr current;

    // Get font height
    SInt16 height = graphics->GetFontHeight();
    // Get the bottom line of current rectangle
    SInt16 bottom = rectangle.GetBottom();
    // Set rectangle height to that of font
    rectangle.SetHeight(height);

    // Call parent handler ( clear screen )
    SFRTitleWindow::ContentHandler(graphics);
    // Get current feed from application class
    current = RssReader::GetCurrentFeed();

    rectangle.AddY((6 * height + 15));

    // Set the height of rectangle as a function of font height
    rectangle.SetHeight(((bottom - rectangle.GetTop()) / height) * height);
    graphics->FillRectangle(rectangle, SFXRGBColor(255, 255, 255, 0));

    // Set the character color
    graphics->SetForeColor(SFXRGBColor(0, 0, 0, 0));
    if (current != null &&
            current->GetSize() > _displine + _cursor) {
        _currentRect = rectangle;
        if(_currentRect.GetY() >= rectangle.GetY())
        {
            _currentRect.SubY(_moveY);
        }
        _currentRect.AddHeight(500);
        // Draw the description of item that _cursor points to
        graphics->DrawString(
            current->GetItem(_displine + _cursor)->description,
            _currentRect,
            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    }


    rectangle.SubY((6 * height + 15));
    // Set rectangle height to that of font
    rectangle.SetHeight(height);

    // Draw each line
    for (i = _displine;  i < _displine + _maxline ; ++i) {
        // If it forms a line with the cursor
        if (i - _displine == _cursor) {
            // Change color
            forecolor.Set(0xff, 0xff, 0xff, 0x00);
            backcolor.Set(0x66, 0x66, 0xff, 0x00);
        }
        else {
            forecolor.Set(0x00, 0x00, 0x00, 0x00);
            // The color is changed at odd or even lines
            if (i % 2 == 0) {
                backcolor.Set(0xdd, 0xdd, 0xff, 0);
            } else {
                backcolor.Set(0xee, 0xee, 0xff, 0);
            }
        }
        // Draw background rectangle of item title
        graphics->FillRectangle(rectangle, backcolor);
        graphics->SetForeColor(forecolor);
        if (current != null &&
                i < current->GetSize()) {
            if (current->GetItem(i)->title.IsEmpty()) {
                // The title strings of items are drawn
                graphics->DrawText("[No title]",
                    rectangle,
                    IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
            } else {
                // If it forms a line with the cursor
                if (i - _displine == _cursor) {
                    _currentRect = rectangle;
                    _currentRect.SubX(_moveX);
                    _currentRect.AddWidth(150);
                    if (current->GetItem(i)->alreadyRead) {
                        // Draw title string of i-th item
                        graphics->DrawText(
                            current->GetItem(i)->title,
                            _currentRect,
                            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
                    } else {
                        // Set the font color
                        SFXRGBColor fontColor(0x99, 0x00, 0x00, 0x00);
                        graphics->SetFont(AEE_FONT_BOLD);
                        // Draw title string of i-th item
                        graphics->DrawText(
                            current->GetItem(i)->title,
                            _currentRect, fontColor,
                            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
                        graphics->SetFont(AEE_FONT_NORMAL);
                    }
                }
                else{
                    if (current->GetItem(i)->alreadyRead) {
                        // Draw title string of i-th item
                        graphics->DrawText(
                            current->GetItem(i)->title,
                            rectangle,
                            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
                    } else {
                        // Set the font color
                        SFXRGBColor fontColor(0x99, 0x00, 0x00, 0x00);
                        graphics->SetFont(AEE_FONT_BOLD);
                        // Draw title string of i-th item
                        graphics->DrawText(
                            current->GetItem(i)->title,
                            rectangle, fontColor,
                            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
                        graphics->SetFont(AEE_FONT_NORMAL);
                    }
                }
            }
        }
        rectangle.AddY(height);
    }
    rectangle.SubY(3);
    rectangle.Deflate(0,3);
    graphics->FillRectangle(rectangle, SFXRGBColor(128, 128, 128, 255));

    return;
}

//
// Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(MainWindow, OnKey, event)
{
    SFXRectangle rectangle;
    RSSFeedPtr current;

    switch (event.GetP16()) {
        // If down key pressed
        case AVK_DOWN:
            // Move display line down
            if (_cursor == _maxline - 1) {
                ++_displine;
                InvalidateContent();
            } else {
                // Move cursor down
                ++_cursor;
                // Redraw window
                InvalidateContent();
            }
            // Reset description rectangle
            _moveY = 0;
            return true;

        // If up key pressed
        case AVK_UP:
            // If cursor is at the top
            if (_cursor == 0) {
                // If display line can be moved up
                if (_displine > 0) {
                    // Move display line up
                    --_displine;
                    // Redraw window
                    InvalidateContent();
                }
            } else {
                // Move cursor up
                --_cursor;
                // Redraw window
                InvalidateContent();
            }
            // Reset description rectangle
            _moveY = 0;
            return true;

        // Scroll horizontally right
        case AVK_RIGHT:
            _moveX = _moveX + 5;
            InvalidateContent();
            return true;

        // Scroll horizontally left
        case AVK_LEFT:
            if(_moveX > 0)
            {
                _moveX = _moveX - 5;
                InvalidateContent();
            }
            return true;

        // Scroll vertically down
        case AVK_8:
            _moveY = _moveY + 5;
            InvalidateContent();
            return true;

        // Scroll vertically up
        case AVK_2:
            if(_moveY > 0)
            {
                _moveY = _moveY - 5;
                InvalidateContent();
            }
            return true;

        case AVK_5:
            RSSFeedPtr current_item;
            current_item = RssReader::GetCurrentFeed();
            if (current_item != null && current_item->GetSize() > _displine + _cursor){
                ::new DetailWindow(_cursor);
            }
            return true;

        // If SOFT2 or 9 key pressed
        case AVK_SOFT2:
        case AVK_9:
            // Create window for managing feeds
            rectangle.Set(GetBaseBound());
            rectangle.AddLeftTop(SUBSCRIPTION_MARGIN, SUBSCRIPTION_MARGIN);
            ::new SubscriptionWindow(rectangle);
            return true;

        // If SELECT key pressed
        case AVK_SELECT:
            current = RssReader::GetCurrentFeed();
            // Change staus of items from unread to already read
            if (current != null &&
                    current->GetSize() > _displine + _cursor) {
                current->GetItem(_displine + _cursor)->alreadyRead = true;
                InvalidateContent();
            }
            return true;
        }
    return false;
}
