#ifndef __RSSDATA_HPP
#define __RSSDATA_HPP

#include <SophiaFramework.hpp>

#define DELIMITER_STR "<>"
#define DELIMITER_LEN 2

#define DIRECTORY_NAME "/data/"
#define DIRECTORY_PATH (SFXPath(DIRECTORY_NAME))
#define LIST_FILE_NAME "list.txt"

// RSSItem structure (one item)
SFMTYPEDEFSTRUCT(RSSItem)
struct RSSItem {
    SFXAnsiString title;
    SFXAnsiString description;
    // For saving date in format of SFXDate
    SFXDate date;
    // Flag showing whether it has been read or not
    Bool alreadyRead;

    RSSItem(SFXAnsiStringConstRef rsstitle,
        SFXAnsiStringConstRef rssdescription,
        SFXDateConstRef rssdate,
        Bool rssalreadyRead = false);
};

// RSSfeed class (one RSS feed)
SFMTYPEDEFCLASS(RSSFeed)

class RSSFeed {
public:

    // Site title
    SFXAnsiString _title;
    // Site URL
    SFXAnsiString _url;
    // Latest updated date
    SFXDate _date;

private:

    // Name of file to save RSS data
    SFXAnsiString _filename;
    // Two or more RSS items are saved
    SFXList<RSSItemPtr> _itemList;

public:

    RSSFeed(SFXAnsiStringConstRef title,
    SFXAnsiStringConstRef url, SFXDateConstRef date = SFXDate());
    ~RSSFeed(Void);
    SFCError Parse(SFXAnsiStringConstRef xml);
    SFCError SaveToFile(Void);
    Void LoadFromFile(Void);
    Bool Find(RSSItemConstRef item);
    RSSItemPtr GetItem(SInt32 index) { return _itemList.Get(index); }
    SFXAnsiStringConstRef GetFilename(Void) { return _filename; }
    Void SetFilename(SFXAnsiStringConstRef filename) { _filename = filename; }
    SInt32 GetSize(Void) { return _itemList.GetSize(); }

    friend class RSSFeedList;
};

// RSSFeedList class (list of RSS feeds)
SFMTYPEDEFCLASS(RSSFeedList)

class RSSFeedList {
private:

    // Array where RSS feeds are stored
    SFXArray<RSSFeedPtr> _feedArray;

public:

    ~RSSFeedList(Void);
    Bool Add(RSSFeedPtr feed);
    SFCError SaveToFile(Void);
    Void LoadFromFile(Void);
    SInt32 GetSize(Void) { return _feedArray.GetSize(); }
    SInt32 FirstIndexOf(RSSFeedPtr item) { return _feedArray.FirstIndexOf(item); }
    RSSFeedPtr operator[](SInt32 index) { return _feedArray[index]; }
};

#endif //__RSSDATA_HPP
