
#include "SoftkeyWindow.hpp"

#define SOFTKEY_MARGIN 2

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
SoftkeyWindow::SoftkeyWindow(Void) : SFRPlainWindow(
    SFRApplication::GetInstance(),
    SFXGraphics::GetDeviceRectangle().SetTop(
        SFXGraphics::GetDeviceRectangle().GetBottom() - GetHeight()),
    BEHAVIOR_SFRPLAINWINDOW, ATTRIBUTE_SOFTKEY) static_throws
{
    // Drawing handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE,
                HANDLER_FUNCTION(OnRenderContent)));
    }
    return;
}

//
// Returns the softkey interface.
//
SoftkeyWindowPtr SoftkeyWindow::GetInstance(Void)
{
    return static_cast<SoftkeyWindowPtr>(SFRApplication::GetInstance()
        ->GetFront(TYPE_SFRWINDOW, ATTRIBUTE_SOFTKEY, BEHAVIOR_NONE));
}

//
// Returns the softkey height.
//
SInt16 SoftkeyWindow::GetHeight(Void)
{
    return SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + SOFTKEY_MARGIN;
}

//
// Set the softkey strings.
//
Void SoftkeyWindow::Set(SFXAnsiStringConstRef left, SFXAnsiStringConstRef center, SFXAnsiStringConstRef right)
{
    _left = left;
    _center = center;
    _right = right;

    // Redraw
    InvalidateContent();
    return;
}

//
// Softkey drawing handler.
//
HANDLER_IMPLEMENT_VOIDRENDER(SoftkeyWindow, OnRenderContent, graphics)
{
    SFXRectangle rectangle(GetContentWorld());
    SInt16 width = rectangle.GetRight();

    // Color constant
    static SFXRGBColor::AtomRecConst black = {0x00, 0x00, 0x00, 0x00};
    static SFXRGBColor::AtomRecConst inner = {0x00, 0xdd, 0xdd, 0xff};
    static SFXRGBColor::AtomRecConst outer = {0x00, 0xee, 0xee, 0xff};

    // The outside frame
    graphics->FillRectangle(rectangle, outer);
    rectangle.DivRight(3);
    rectangle.Deflate(1, 1);

    // The inside frame
    graphics->FillRectangle(rectangle, inner);

    // Left string
    if (!_left.IsEmptyCString()) {
        graphics->DrawText(_left, rectangle, black, IDF_ALIGN_CENTER);
    }
    rectangle.SetX(width / 3 + 1);
    graphics->FillRectangle(rectangle, inner);

    // Center string
    if (!_center.IsEmptyCString()) {
        graphics->DrawText(_center, rectangle, black, IDF_ALIGN_CENTER);
    }
    rectangle.SetX(width * 2 / 3 + 1);
    graphics->FillRectangle(rectangle, inner);

    // Right string
    if (!_right.IsEmptyCString()) {
        graphics->DrawText(_right, rectangle, black, IDF_ALIGN_CENTER);
    }
    return;
}
