#include "RssReader.hpp"
#include "SubscriptionWindow.hpp"
#include "MainWindow.hpp"
#include "InputWindow.hpp"
#include "SoftkeyWindow.hpp"

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
SubscriptionWindow::SubscriptionWindow(SFXRectangleConstRef rectangle) :
    SFRTitleWindow(SFRApplication::GetInstance(),
    rectangle, "Feed List") static_throws
{
    // Drawing handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT,
                HANDLER_BEFORE, HANDLER_FUNCTION(OnRenderContent)));
    }

    // Key handler registration
    if (static_try()) {
        static_throw(
            RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }
    SetSoftkey();

    // Initialize the cursor position
    _displine = static_cast<SInt16>(RssReader::GetRSSFeedList()->FirstIndexOf(RssReader::GetCurrentFeed()));
    if (_displine > 0) {
        --_displine;
        _cursor = 1;
    } else {
        _displine = 0;
        _cursor = 0;
    }
    // Number of display lines calculation
    _maxline = GetContentWorld().GetHeight()/SFXGraphics::GetFontHeight(AEE_FONT_NORMAL)/2;

    _move = 0;
    return;
}

//
// Set the softkey strings.
//
Void SubscriptionWindow::SetSoftkey(Void)
{
    SoftkeyWindow::GetInstance()->Set("Update", "Select", "Add");
    return;
}

//
// Window drawing handler.
//
HANDLER_IMPLEMENT_VOIDRENDER(SubscriptionWindow, OnRenderContent, graphics)
{
    // Get screen size
    SFXRectangle rectangle(GetContentWorld());
    SFXRGBColor forecolor;
    SFXRGBColor backcolor;
    SFXAnsiString buff;
	SIntN i;

    // Get the font height
    SInt16 height = graphics->GetFontHeight();
    // Get the rectangular base
    SInt16 bottom = rectangle.GetBottom();
    // The rectangle height is set to font height
    rectangle.SetHeight(height);

    SFRTitleWindow::ContentHandler(graphics);
    for (i = _displine;  i < _displine + _maxline ; ++i) {
        // If it forms a line with the cursor
        if (i - _displine == _cursor) {
            forecolor.Set(0xff, 0xff, 0xff, 0x00);
            backcolor.Set(0x66, 0x66, 0xff, 0x00);
        }
        else {
            forecolor.Set(0x00, 0x00, 0x00, 0x00);
            // The color is changed at odd or even lines
            if (i % 2 == 0) {
                backcolor.Set(0xdd, 0xdd, 0xff, 0);
            } else {
                backcolor.Set(0xee, 0xee, 0xff, 0);
            }
        }
        graphics->FillRectangle(rectangle, backcolor);
        graphics->SetForeColor(forecolor);
        if (i < RssReader::GetRSSFeedList()->GetSize()) {
            if ((*RssReader::GetRSSFeedList())[i]->_title.IsEmptyCString()) {
                // The feed strings of items are drawn
                graphics->DrawText("[No title]",
                    rectangle,
                    IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
            } else {
                // If it forms a line with the cursor
                if (i - _displine == _cursor) {
                _currentRect = rectangle;
                _currentRect.SubX(_move);
                // The feed strings of items are drawn
                graphics->DrawText((*RssReader::GetRSSFeedList())[i]->_title,
                    rectangle,
                    IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);}
                else{
                // The feed strings of items are drawn
                graphics->DrawText((*RssReader::GetRSSFeedList())[i]->_title,
                    rectangle,
                    IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);}

            }
        }
        rectangle.AddY(height);
    }
    rectangle.AddY(15);

    // The rectangle height is a multiple of the font height
    rectangle.SetHeight(((bottom - rectangle.GetTop()) / height) * height);
    graphics->FillRectangle(rectangle, SFXRGBColor(255, 255, 255, 0));
    graphics->SetForeColor(SFXRGBColor(0, 0, 0, 0));
    if (RssReader::GetRSSFeedList()->GetSize() > _displine + _cursor) {
        // Item description that cursor points to is drawn
		buff= "URL :" + (*RssReader::GetRSSFeedList())[_displine + _cursor]->_url + "\r\n";
        graphics->DrawString(
            buff,
            rectangle,
            IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    }

    // Get rectangle's top line
    SFXLine line(rectangle.GetEdgeTop());
    graphics->DrawLine(line, SFXRGBColor(128, 128, 128, 255));
    line.Unoffset(0, 10);
    graphics->DrawLine(line, SFXRGBColor(128, 128, 128, 255));
    return;
}

//
// Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(SubscriptionWindow, OnKey, event) static_throws
{
    SFXRectangle rectangle;
    SFRMultiDialogPtr dialog;
    SFRMultiDialog::ParamRec dialogParam;
    RSSFeedListRef feedList = *RssReader::GetRSSFeedList();

    switch (event.GetP16()) {
        case AVK_DOWN:
            // If the cursor is at the bottom of screen
            if (_cursor == _maxline - 1) {
                ++_displine;
                InvalidateContent();
            } else {
                ++_cursor;
                InvalidateContent();
            }
            UpdateMainWindow();
            return true;

        case AVK_UP:
            // If the cursor is at the top of screen
            if (_cursor == 0) {
                if (_displine > 0) {
                    --_displine;
                    InvalidateContent();
                }
            } else {
                --_cursor;
                InvalidateContent();
            }
            UpdateMainWindow();
            return true;

        // Right key
        case AVK_RIGHT:
            _move++;
            InvalidateContent();
            return true;

        // Left key
        case AVK_LEFT:
            if(_move > 0)
            {
                _move--;
                InvalidateContent();
            }
            return true;

        case AVK_SOFT1:
        case AVK_7:
            RssReader::SetCurrentFeed(feedList[_displine + _cursor]);

            // Connection confirmation dialog is displayed
            dialogParam.text = "Would you like to connect now?";
            dialogParam.ok = "Yes";
            dialogParam.cancel = "No";
            dialogParam.another = "";
            dialogParam.focus = SFRMultiDialog::FOCUS_OK,
            dialogParam.image = SFBImageSmp(null);

            // Set the dialog size
            rectangle.Set(SFXGraphics::GetDeviceRectangle());
            rectangle.Deflate(10, 50).SetHeight(150);

            dialog = ::new SFRMultiDialog(rectangle, "Connect", dialogParam);
            if (dialog != null && static_try()) {
            // Handler registration
                static_throw(
                    dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE,
                        HANDLER_FUNCTION(StartConnection)));
            }
            return true;

        case AVK_SELECT:
        case AVK_CLR:
            // Select Feed
            Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,
                SRP16_TERMINATE_INVOKE, true));
            MainWindow::SetSoftkey();
            return true;

        case AVK_SOFT2:
        case AVK_9:
            // Open the feed adding window
            ::new InputWindow();
            InputWindow::SetSoftkey();
            return true;
    }
    return false;
}

//
// When the cursor is moved, draw again the MainWindow.
//
Void SubscriptionWindow::UpdateMainWindow(Void)
{
    SFXAnsiString windowTitle;
    MainWindowPtr mainWindow;
    RSSFeedListRef feedList = *RssReader::GetRSSFeedList();

    if (feedList.GetSize() > _displine + _cursor) {
        RssReader::SetCurrentFeed(feedList[_displine + _cursor]);
        windowTitle = RssReader::GetCurrentFeed()->_title;
    } else {
        RssReader::SetCurrentFeed(null);
        windowTitle = SFXAnsiString::EmptyInstance();
    }
    // Get MainWindow
    mainWindow = static_cast<MainWindowPtr>(GetNext());

    // Set MainWindow title
    mainWindow->SetTitle(windowTitle);

    // Redraw MainWindow
    mainWindow->InvalidateBase();
    mainWindow->InvalidateContent();
    return;
}

HANDLER_IMPLEMENT_VOIDDIALOG(SubscriptionWindow, StartConnection, result, dialog) static_throws
{
    SFCError error;
    SFXRectangle rectangle;
    SFRDialogPtr connectingDialog;

    switch(result) {
        case SRP16_OK:
            // First call Open()
            error = _http.Open();
            if (error == SFERR_NO_ERROR) {
                error = _http.SetMethod("GET");
                if (error == SFERR_NO_ERROR) {

                    // Start to connect by calling Connect()
                    // 1st argument: URL string
                    // 2nd argument: callback function on http connection
                    error = _http.Connect(
                        (*RssReader::GetRSSFeedList())[_displine
                            + _cursor]->_url,
                        CALLBACK_FUNCTION(ConnectCallback));

                    if (error == SFERR_NO_ERROR) {
                        // Set dialog outer rectangle
                        rectangle.Set(SFXGraphics::GetDeviceRectangle());
                        rectangle.Deflate(10, 100);
                        // Display the "connecting..." dialog
                        connectingDialog = ::new SFRMessageDialog(rectangle,
                            "Connecting...", "");
                        if (connectingDialog != null && static_try()) {
                            // The input key is not accepted
                            static_throw(
                                connectingDialog->RegisterHandler(SFEVT_KEY,
                                    HANDLER_AFTER,
                                    HANDLER_NULL));
                        }
                        SoftkeyWindow::GetInstance()->Set("");
                    }
                }
            }
            if (error != SFERR_NO_ERROR) {
                DisplayErrorDialog();
            }
            break;
        case SRP16_CANCEL:
            SetSoftkey();
            break;
    }
    dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,
        SRP16_TERMINATE_INVOKE, true));
    return;
}

Void SubscriptionWindow::DisplayErrorDialog(Void)
{
    SFXRectangle rectangle;
    // The window in front position is closed
    _http.Close();

    SFRResponderPtr dialog = SFRApplication::GetInstance()
        ->GetFront(TYPE_WILDCARD, ATTRIBUTE_WILDCARD);
    if (dialog == null) {
        dialog = SFRApplication::GetInstance()
            ->GetFront(TYPE_WILDCARD, ATTRIBUTE_WILDCARD);
    }

    if (dialog != null) {
        dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,
            SRP16_TERMINATE_INVOKE, true));
    }
    rectangle.Set(SFXGraphics::GetDeviceRectangle());
    rectangle.Deflate(10, 100);
    ::new SFRMessageDialog(rectangle, "Connection Failed", "OK");

    // Redraw
    SFRApplication::GetInstance()->Invoke(
        SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false));
    return;
}

//
// Callback that notifies connection confirmation.
//
CALLBACK_IMPLEMENT_SFXHTTPCONNECTION(SubscriptionWindow, ConnectCallback, error)
{
    if (error == SFERR_NO_ERROR) {
        // Get stream for reading data
        error = _http.GetStreamReader(1024, &_reader);
        if (error == SFERR_NO_ERROR) {
            // Receive data
            error = _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
        }
    }
    if (error != SFERR_NO_ERROR) {
        DisplayErrorDialog();
    }
    return;
}

//
// Callback that notifies data receiving.
//
CALLBACK_IMPLEMENT_SFXBINARYSTREAMREADER(SubscriptionWindow, FetchCallback, error)
{
    SFRResponderPtr dialog;
    SFXAnsiString string;
    RSSFeedPtr feed = (*RssReader::GetRSSFeedList())[_displine + _cursor];

    if (error == SFERR_NO_ERROR) {
        // Read string
        _reader.ReadSFXAnsiString(&string);
        // Append string to the end of _xml
        _xml += string;
        // If receiving is over
        if (_reader.Ends() && _reader.GetReadableSize() == 0) {

            // Close SFXHTTPConnection
            _http.Close();

            // Get current date
            feed->_date = SFXDate::CurrentDate();
            // Parse string
            error = feed->Parse(_xml);
            _xml.Clear();

            if (error == SFERR_NO_ERROR) {
                // Save results to file
                feed->SaveToFile();
                // Save to list
                RssReader::GetRSSFeedList()->SaveToFile();
            }

            // Get the front window
            dialog = SFRApplication::GetInstance()->GetFront(TYPE_WILDCARD);
            if (dialog != null) {
                // Close window
                dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,
                    SRP16_TERMINATE_INVOKE, true));
            }
            SetSoftkey();
            // Redraw
            SFRApplication::GetInstance()->Invoke(
                SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
        }
        else {
            // If there still remains data to be received
            _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
        }
    }
    else {
        DisplayErrorDialog();
    }
    return;
}
